
;       Written to function from within MASM programs compiled with Ver. 4.0+
;       and modified from Sam Smith's original code.  11-01-91 John Van Lund

;       Calculates an accumulated byte total for 16 bit CRC's and stores
;       it in the word variable Sav_Crc - (Hex word 2 Byte) CRC values are
;       calculated based on the IBM BySync polynomial integer Hex (8005h)

;       Modifies Registers Ax,Bx,Cx,Dx,Si,Di - So we save them first


Calculate_CRC   Proc    Near

        Push    Ax                      ;Save contents of the Ax register
        Push    Bx
        Push    Cx
        Push    Dx
        Push    Si
        Push    Di

;       The routine expects input from Ax,Si - In this example a null
;       terminated string.

        Mov     Si,OffSet MyString      ;Point Si to the Offset for MyString
        Mov     Cx,StrLen               ;Load Cx w/bytes equal MyString Length
        Dec     Cx                      ;Length includes null, so strip it off

        Mov     Ax,0                    ;Zero (0) the Ax register
        Mov     Bx,0                    ;Zero (0) the Bx register
        Mov     Dx,0                    ;Zero (0) the Dx register

;       Perform Crc calculations

NextCrc:

        Mov Al,[Si]                     ;Load a byte from String Offset
        Inc Si                          ;Bump Si forward one count
        Xor Ah,Ah                       ;Zero the High side of Ax (Ah)
        Xor Ax,Dx                       ;Rotate bits in Ax * Dx
        Xor Ah,Ah                       ;Zero the High side of Ax (Ah)
        Mov Di,Ax                       ;Place a copy of Ax into Di
        Shl Di,1                        ;Shift bits in Di to the left (1)
        Mov Bx,offset [Di]+[Crc_Table]  ;Load Bx with DI+table offset value
        Mov Al,Dh                       ;Copy the high side of Dx (Dh) to Al
        Xor Ah,Ah                       ;Zero the High side of Ax (Ah)
        Xor Ax,Bx                       ;Rotate bits in Ax * Bx (Ax=Crc)
        Mov Dx,Ax                       ;Dx is the Crc "accumulator"

        Dec Cx                          ;When Cx=(0) all bytes calculated
        Jnz     NextCrc                 ;If Cx >0 then more bytes to process

        Mov     Sav_Crc,Dx              ;Save calculated Crc value

        Pop     Di                      ;Restore registers to original content
        Pop     Si
        Pop     Dx
        Pop     Cx
        Pop     Bx
        Pop     Ax

        Ret                             ;Return to point of call(er)


;       Sam's Crc Table used for Very fast calculation of Crc 16 bit values


CRC_TABLE DW   0000h, 0C0C1h, 0C181h, 0140h, 0C301h, 03C0h, 0280h, 0C241h

          DW   0C601h, 06C0h, 0780h, 0C741h, 0500h, 0C5C1h, 0C481h, 0440h

          DW   0CC01h, 0CC0h, 0D80h, 0CD41h, 0F00h, 0CFC1h, 0CE81h, 0E40h

          DW   0A00h,  0CAC1h,0CB81h,0B40h,  0C901h,09C0h,  0880h,  0C841h

          DW   0D801h, 18C0h, 1980h, 0D941h, 1B00h, 0DBC1h, 0DA81h, 1A40h

          DW   1E00h,  0DEC1h,0DF81h,1F40h,  0DD01h,1DC0h,  1C80h,  0DC41h

          DW   1400h,  0D4C1h,0D581h,1540h,  0D701h,17C0h,  1680h,  0D641h

          DW   0D201h, 12C0h, 1380h, 0D341h, 01100h,0D1C1h, 0D081h, 1040h

          DW   0F001h, 30C0h, 3180h, 0F141h, 3300h, 0F3C1h, 0F281h, 3240h

          DW   3600h,  0F6C1h,0F781h,03740h, 0F501h,35C0h,  3480h,  0F441h

          DW   3C00h,  0FCC1h,0FD81h,3D40h,  0FF01h,3FC0h,  3E80h,  0FE41h

          DW   0FA01h, 3AC0h, 3B80h, 0FB41h, 3900h, 0F9C1h, 0F881h, 3840h

          DW   2800h,  0E8C1h,0E981h,2940h,  0EB01h,2BC0h,  2A80h,  0EA41h

          DW   0EE01h, 2EC0h, 2F80h, 0EF41h, 2D00h, 0EDC1h, 0EC81h, 2C40h

          DW   0E401h, 24C0h, 2580h, 0E541h, 2700h, 0E7C1h, 0E681h, 2640h

          DW   2200h,  0E2C1h,0E381h,2340h,  0E101h,21C0h,  2080h,  0E041h

          DW   0A001h, 60C0h, 6180h, 0A141h, 6300h, 0A3C1h, 0A281h, 6240h

          DW   6600h,  0A6C1h,0A781h,6740h,  0A501h,65C0h,  6480h,  0A441h

          DW   6C00h,  0ACC1h,0AD81h,6D40h,  0AF01h,6FC0h,  6E80h,  0AE41h

          DW   0AA01h, 6AC0h, 6B80h, 0AB41h, 6900h, 0A9C1h, 0A881h, 6840h

          DW   7800h,  0B8C1h,0B981h,7940h,  0BB01h,7BC0h,  7A80h,  0BA41h

          DW   0BE01h, 7EC0h, 7F80h, 0BF41h, 7D00h, 0BDC1h, 0BC81h, 7C40h

          DW   0B401h, 74C0h, 7580h, 0B541h, 7700h, 0B7C1h, 0B681h, 7640h

          DW   7200h,  0B2C1h,0B381h,7340h,  0B101h,71C0h,  7080h,  0B041h

          DW   5000h,  90C1h, 9181h, 5140h,  9301h, 53C0h,  5280h,  9241h

          DW   9601h,  56C0h, 5780h, 9741h,  5500h, 95C1h,  9481h,  5440h

          DW   9C01h,  5CC0h, 5D80h, 9D41h,  5F00h, 9FC1h,  9E81h,  5E40h

          DW   5A00h,  9AC1h, 9B81h, 5B40h,  9901h, 59C0h,  5880h,  9841h

          DW   8801h,  48C0h, 4980h, 8941h,  4B00h, 8BC1h,  8A81h,  4A40h

          DW   4E00h,  8EC1h, 8F81h, 4F40h,  8D01h, 4DC0h,  4C80h,  8C41h

          DW   4400h,  84C1h, 8581h, 4540h,  8701h, 47C0h,  4680h,  8641h

          DW   8201h,  42C0h, 4380h, 8341h,  4100h, 81C1h,  8081h,  4040h


Calc_CRC        Endp                    ;End of program marker for Calc_Crc

Sav_CRC         Dw      0

Sav_CRC_Check   Dw      0

MyString        Db      'Byte',0

StrLen          Equ      $ - MyString


